package info.textgrid.middleware;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.classicmayan.tools.ClassicMayanMetsMods;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @since 2021-09-09
 * @version 2019-03-12
 */
@Component
public class RecordDelivererIDIOM extends RecordDelivererAbstract {

  private static Log log = LogFactory.getLog(RecordDelivererIDIOM.class);

  private String dateOfLastOblectModification;

  RecordType record = new RecordType();
  ClassicMayanMetsMods metsmods;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * ElasticSearch request in non-public index to get the datestamps for the object creation and
   * last modification date in TextGrid-Repo.
   * </p>
   * 
   * @param idInDatabase
   * @throws ParseException
   * @throws IOException
   */
  public void setDatestamps(String idInDatabase) throws ParseException, IOException {

    String changedId = idInDatabase;

    String[] includes = new String[] {TGConstants.CREATED, TGConstants.MODIFIED_FIELD};
    String[] excludes = Strings.EMPTY_ARRAY;
    FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, excludes);

    if (idInDatabase.startsWith("textgrid:")) {
      changedId = idInDatabase.substring("textgrid:".length());
    }

    if (!idInDatabase.contains(".0")) {
      // changedId = changedId.substring(0,changedId.length() - 1) + "0";
      changedId = changedId + ".0";
    }

    GetRequest getRequest =
        new GetRequest("textgrid-nonpublic", OAI_ESClient.getEsType(), changedId)
            .fetchSourceContext(fetchSourceContext);

    GetResponse objectInDatabase = null;
    try {
      objectInDatabase = OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    JSONObject json = new JSONObject();
    String[] fields = {TGConstants.CREATED, TGConstants.MODIFIED_FIELD};
    json = new JSONObject(OAIPMHUtilities
        .getRcordByIDFromElasticSearch("textgrid-nonpublic", changedId, fields, Strings.EMPTY_ARRAY)
        .getSource());

    //log.debug("id in elasticsearch is: " + OAIPMHUtilities
    //    .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED)));
    //log.debug(json);

    this.dateOfObjectCreation = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED));

    this.dateOfLastOblectModification = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.MODIFIED_FIELD));
    // objectInDatabase.getField(TGConstants.MODIFIED_FIELD).getValue().toString();
  }

  /**
   * building the record XML object for the OAI-PMH response
   * 
   * @throws ParseException
   * @throws IOException
   */
  public GetRecordType getRecordById(String id) throws ParseException, IOException {

    GetRecordType grt = new GetRecordType();

    setDatestamps(id);
    String textgridBaseURI_FromID;
    if (!id.startsWith("textgrid:")) {
      textgridBaseURI_FromID = "textgrid:" + id.replace(".0", "");
    } else {
      textgridBaseURI_FromID = id;
    }

    log.debug("ID for generating the METS/MODS file is: " + textgridBaseURI_FromID);

    this.metsmods = new ClassicMayanMetsMods(
        textgridBaseURI_FromID,
        this.dateOfObjectCreation,
        this.dateOfLastOblectModification);
    this.record.setMetadata(idiomMets());

    String identifierToSet = "";
    if (!id.startsWith("textgrid:")) {
      identifierToSet = "textgrid:" + id;
    } else {
      identifierToSet = id;
    }

    // No setSpec needed here!
    String setSpec = "";
    HeaderType header = OAIPMHUtilities.computeTheF______Header(this.dateOfObjectCreation,
        identifierToSet, setSpec);
    this.record.setHeader(header);

    grt.setRecord(this.record);

    return grt;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * <p>
   * Getting the Mets/Mods object for the ClassicMayan artifact and put them into metadata element
   * of the OAI-PMH response.
   * </p>
   * 
   * @return
   */
  private MetadataType idiomMets() {

    MetadataType metadataMets = new MetadataType();
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;

    try {
      builder = factory.newDocumentBuilder();
      Document doc = builder.parse(new InputSource(new StringReader(this.metsmods.getMets())));
      metadataMets.setAny(doc.getDocumentElement());
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    return metadataMets;
  }

}
